@@ -55,7 +55,7 @@ typedef struct {
 	GthFileSource *vfs_source;
 	DataFunc       done_func;
 	gboolean       cancelling;
-	gulong         monitor_event;
+	gulong         entry_points_changed_id;
 	GtkWidget     *filter_combobox;
 	GtkWidget     *tags_entry;
 	GList         *general_tests;
@@ -68,7 +68,7 @@ destroy_dialog (gpointer user_data)
 	DialogData *data = user_data;
 	gboolean    delete_imported;
 
-	g_signal_handler_disconnect (gth_main_get_default_monitor (), data->monitor_event);
+	g_signal_handler_disconnect (gth_main_get_default_monitor (), data->entry_points_changed_id);
 
 	delete_imported = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("delete_checkbutton")));
 	eel_gconf_set_boolean (PREF_PHOTO_IMPORT_DELETE, delete_imported);
@@ -470,7 +470,6 @@ preferences_button_clicked_cb (GtkWidget  *widget,
 	gtk_window_present (GTK_WINDOW (data->preferences_dialog));
 }
 
-
 void
 dlg_photo_importer (GthBrowser *browser,
 		    GFile      *source)
@@ -499,6 +498,7 @@ dlg_photo_importer (GthBrowser *browser,
 	/* Get the widgets. */
 
 	data->dialog = _gtk_builder_get_widget (data->builder, "photo_importer_dialog");
+	_gtk_window_resize_to_fit_screen_height (data->dialog, 580);
 	gth_browser_set_dialog (browser, "photo_importer", data->dialog);
 	g_object_set_data (G_OBJECT (data->dialog), "dialog_data", data);
 
@@ -525,7 +525,7 @@ dlg_photo_importer (GthBrowser *browser,
 					"text", SOURCE_LIST_COLUMN_NAME,
 					NULL);
 
-	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL);
+	data->file_list = gth_file_list_new (GTH_FILE_LIST_TYPE_NORMAL, FALSE);
 	sort_type = gth_main_get_sort_type ("file::mtime");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (data->file_list), sort_type->cmp_func, FALSE);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->file_list), TRUE);
@@ -615,10 +615,10 @@ dlg_photo_importer (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (preferences_button_clicked_cb),
 			  data);
-	data->monitor_event = g_signal_connect (gth_main_get_default_monitor (),
-						"entry_points_changed",
-						G_CALLBACK (entry_points_changed_cb),
-						data);
+	data->entry_points_changed_id = g_signal_connect (gth_main_get_default_monitor (),
+							  "entry-points-changed",
+							  G_CALLBACK (entry_points_changed_cb),
+							  data);
 
 	/* Run dialog. */
 
