@@ -239,6 +239,14 @@ gth_browser_activate_action_view_sidebar (GtkAction  *action,
 }
 
 
+void
+gth_browser_activate_action_view_thumbnail_list (GtkAction  *action,
+						 GthBrowser *browser)
+{
+	eel_gconf_set_boolean (PREF_UI_THUMBNAIL_LIST_VISIBLE, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+}
+
+
 void
 gth_browser_activate_action_view_stop (GtkAction  *action,
 				       GthBrowser *browser)
@@ -275,11 +283,9 @@ void
 gth_browser_activate_action_folder_open (GtkAction  *action,
 					 GthBrowser *browser)
 {
-	GtkWidget   *folder_tree;
 	GthFileData *file_data;
 
-	folder_tree = gth_browser_get_folder_tree (browser);
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (folder_tree));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -293,12 +299,10 @@ void
 gth_browser_activate_action_folder_open_in_new_window (GtkAction  *action,
 						       GthBrowser *browser)
 {
-	GtkWidget   *folder_tree;
 	GthFileData *file_data;
 	GtkWidget   *new_browser;
 
-	folder_tree = gth_browser_get_folder_tree (browser);
-	file_data = gth_folder_tree_get_selected (GTH_FOLDER_TREE (folder_tree));
+	file_data = gth_browser_get_folder_popup_file_data (browser);
 	if (file_data == NULL)
 		return;
 
@@ -317,7 +321,7 @@ gth_browser_activate_action_browser_mode (GtkAction  *action,
 	GtkWidget *viewer_sidebar;
 
 	viewer_sidebar = gth_browser_get_viewer_sidebar (browser);
-	if (gth_sidebar_is_tool_active (GTH_SIDEBAR (viewer_sidebar)))
+	if (gth_sidebar_tool_is_active (GTH_SIDEBAR (viewer_sidebar)))
 		gth_sidebar_deactivate_tool (GTH_SIDEBAR (viewer_sidebar));
 	else
 		gth_window_set_current_page (GTH_WINDOW (browser), GTH_BROWSER_PAGE_BROWSER);
@@ -410,5 +414,5 @@ void
 gth_browser_activate_action_help_shortcuts (GtkAction *action,
 					    gpointer   data)
 {
-	show_help_dialog (GTK_WINDOW (data), "shortcuts");
+	show_help_dialog (GTK_WINDOW (data), "gthumb-shortcuts");
 }
