@@ -408,7 +408,7 @@ gth_main_register_metadata_info (GthMetadataInfo *metadata_info)
 	GthMetadataInfo *info;
 
 	if ((metadata_info->display_name != NULL) && (strstr (metadata_info->display_name, "0x") != NULL))
-		return NULL;
+		metadata_info->flags = GTH_METADATA_ALLOW_NOWHERE;
 
 	g_static_mutex_lock (&metadata_info_mutex);
 
@@ -638,13 +638,21 @@ collect_sort_types (gpointer key,
 }
 
 
+static gboolean
+campare_sort_types (GthFileDataSort *type1,
+		    GthFileDataSort *type2)
+{
+	return g_utf8_collate (type1->display_name, type2->display_name);
+}
+
+
 GList *
 gth_main_get_all_sort_types (void)
 {
 	GList *sort_types = NULL;
 
 	g_hash_table_foreach (Main->priv->sort_types, collect_sort_types, &sort_types);
-	return g_list_reverse (sort_types);
+	return g_list_sort (sort_types, (GCompareFunc) campare_sort_types);
 }
 
 
@@ -1151,10 +1159,9 @@ gth_main_tags_changed (void)
 	gth_user_dir_make_dir_for_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
 	filename = gth_user_dir_get_file (GTH_DIR_CONFIG, GTHUMB_DIR, TAGS_FILE, NULL);
 	gth_tags_file_to_file (Main->priv->tags, filename, NULL);
+	gth_monitor_tags_changed (gth_main_get_default_monitor ());
 
 	g_free (filename);
-
-	gth_monitor_tags_changed (gth_main_get_default_monitor ());
 }
 
 
