@@ -677,6 +677,8 @@ get_themed_icon_pixbuf (GThemedIcon  *icon,
 
 	g_object_get (icon, "names", &icon_names, NULL);
 
+	if (icon_theme == NULL)
+		icon_theme = gtk_icon_theme_get_default ();
 	icon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)icon_names, size, 0);
 	if (icon_info == NULL)
 		icon_info = gtk_icon_theme_lookup_icon (icon_theme, "text-x-generic", size, GTK_ICON_LOOKUP_USE_BUILTIN);
@@ -698,14 +700,20 @@ static GdkPixbuf *
 get_file_icon_pixbuf (GFileIcon *icon,
 		      int        size)
 {
+	GdkPixbuf *pixbuf = NULL;
 	GFile     *file;
-	char      *filename;
-	GdkPixbuf *pixbuf;
 
 	file = g_file_icon_get_file (icon);
-	filename = g_file_get_path (file);
-	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, size, -1, NULL);
-	g_free (filename);
+	if (file != NULL) {
+		char *filename;
+
+		filename = g_file_get_path (file);
+		if (filename != NULL)
+			pixbuf = gdk_pixbuf_new_from_file_at_size (filename, size, -1, NULL);
+
+		g_free (filename);
+	}
+
 	g_object_unref (file);
 
 	return pixbuf;
@@ -1015,21 +1023,31 @@ _gtk_tree_path_list_free (GList *list)
 int
 _gtk_paned_get_position2 (GtkPaned *paned)
 {
-	GtkRequisition requisition;
-	int            pos;
-	int            size;
+	int             pos;
+	GtkWidget      *w;
+	GtkRequisition  requisition;
+	int             size;
 
 	if (! GTK_WIDGET_VISIBLE (paned))
 		return 0;
 
 	pos = gtk_paned_get_position (paned);
+	if (pos == 0)
+		return 0;
 
-	gtk_window_get_size (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (paned))), &(requisition.width), &(requisition.height));
+	w = gtk_widget_get_toplevel (GTK_WIDGET (paned));
+	if (! GTK_WIDGET_TOPLEVEL (w))
+		return 0;
+
+	gtk_window_get_size (GTK_WINDOW (w), &(requisition.width), &(requisition.height));
 	if (gtk_orientable_get_orientation (GTK_ORIENTABLE (paned)) == GTK_ORIENTATION_HORIZONTAL)
 		size = requisition.width;
 	else
 		size = requisition.height;
 
+	if (size == 0)
+		return 0;
+
 	return size - pos;
 }
 
@@ -1042,6 +1060,9 @@ _gtk_paned_set_position2 (GtkPaned *paned,
 	int        size;
 
 	top_level = gtk_widget_get_toplevel (GTK_WIDGET (paned));
+	if (! GTK_WIDGET_TOPLEVEL (top_level))
+		return;
+
 	if (gtk_orientable_get_orientation (GTK_ORIENTABLE (paned)) == GTK_ORIENTATION_HORIZONTAL)
 		size = top_level->allocation.width;
 	else
@@ -1078,3 +1099,20 @@ _g_launch_command (GtkWidget  *parent,
 
 	g_object_unref (app_info);
 }
+
+
+void
+_gtk_window_resize_to_fit_screen_height (GtkWidget *window,
+					 int        default_width)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (window);
+	if ((screen != NULL) && (gdk_screen_get_height (screen) < 768))
+		/* maximize on netbooks */
+		gtk_window_maximize (GTK_WINDOW (window));
+	else
+		/* This should fit on a XGA/WXGA (height 768) screen
+		 * with top and bottom panels */
+		gtk_window_set_default_size (GTK_WINDOW (window), default_width, 670);
+}
